// Very basic Karplus-Strong 

// based on this explanation http://sites.music.columbia.edu/cmc/MusicAndComputers/chapter4/04_09.php


#include <Adafruit_MCP4728.h>
#include <Wire.h>

Adafruit_MCP4728 mcp;

#define nsamp 128
#define dacmax 256

unsigned int phase;
unsigned int repeats;


void setup() {


  cli();

//set timer1 interrupt at 1Hz
  TCCR1A = 0;// set entire TCCR1A register to 0
  TCCR1B = 0;// same for TCCR1B
  TCNT1  = 0;//initialize counter value to 0
  // set compare match register for 1hz increments
  //OCR1A = 15624;// = (16*10^6) / (1*1024) - 1 (must be <65536)
  //OCR1A = 7812;// = (16*10^6) / (1*1024) - 1 (must be <65536)
  OCR1A = 1952;
  // turn on CTC mode
  TCCR1B |= (1 << WGM12);
  // Set CS10 and CS12 bits for 1024 prescaler
  TCCR1B |= (1 << CS12) | (1 << CS10);  
  // enable timer compare interrupt
  TIMSK1 |= (1 << OCIE1A);

  sei();


  Serial.begin(115200);

  if (!mcp.begin(0x64)) {
    while (1) {
      delay(100);
    }
  }

//  mcp.setSpeed(400000L);
//  mcp.setSpeed(800000L);
  mcp.setSpeed(800000L);

  randomSeed(analogRead(A0));

  phase=0;
}

byte waveform[nsamp];
void pluck(){
  for (int i=0; i<nsamp; ++i){
    waveform[i]=random(256);
  }
}

void loop() {
  byte s1, s2;
  int p0 = phase;
  s1 = waveform[phase];
  phase += 1;
  if( phase > nsamp ) {
    phase = 0;
  }
  s2 = waveform[phase];
  waveform[p0] = ( s1 & s2 ) + ((s1 ^ s2) >> 1);
  mcp.fastWrite(s1 << 4, 0, 0, 0);
}

ISR(TIMER1_COMPA_vect){//timer1 interrupt 1Hz 
    pluck();
}

